@@ -1,19 +1,19 @@
 /****
-Copyright (c) 2016, Benjamin Buchfink
-All rights reserved.
+DIAMOND protein aligner
+Copyright (C) 2013-2017 Benjamin Buchfink <buchfink@gmail.com>
 
-Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
+This program is free software: you can redistribute it and/or modify
+it under the terms of the GNU Affero General Public License as
+published by the Free Software Foundation, either version 3 of the
+License, or (at your option) any later version.
 
-1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU Affero General Public License for more details.
 
-2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+You should have received a copy of the GNU Affero General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
 #include "value.h"
@@ -22,8 +22,9 @@ LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR P
 #include "translate.h"
 #include "statistics.h"
 #include "sequence.h"
+#include "masking.h"
 
-const char* Const::version_string = "0.8.35";
+const char* Const::version_string = "0.9.8";
 const char* Const::program_name = "diamond";
 const char* Const::id_delimiters = " \a\b\f\n\r\t\v";
 
@@ -72,8 +73,8 @@ unsigned Align_mode::from_command(unsigned command)
 
 Align_mode align_mode (Align_mode::blastp);
 
-Reduction Reduction::reduction("KREDQN C G H M F Y ILV W P STA");
-//Reduction Reduction::reduction("A KR EDNQ C G H ILVM FYW P ST"); // murphy.10
+//Reduction Reduction::reduction("KREDQN C G H M F Y ILV W P STA");
+Reduction Reduction::reduction("A KR EDNQ C G H ILVM FYW P ST"); // murphy.10
 //const Reduction Reduction::reduction("G D N AEFIKLMQRVW Y H C T S P"); // gmbr.10
 //const Reduction Reduction::reduction("EKQR IV LY F AM W HT C DNS"); // dssp.10
 //const Reduction Reduction::reduction("K R E D Q N C G H M F Y I L V W P S T A");
@@ -81,8 +82,8 @@ Reduction Reduction::reduction("KREDQN C G H M F Y ILV W P STA");
 Statistics statistics;
 
 const char* shape_codes[][Const::max_shapes] = {
-	{ "111101011101111", "111011001100101111", "1111001001010001001111", "111100101000010010010111", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },				// 4x12
-	{ "1111011111",		// 16x9
+	{ "111101011101111", "111011001100101111", "1111001001010001001111", "111100101000010010010111", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },				// 0 4x12
+	{ "1111011111",		// 1 16x9
 	"111001101111",
 	"11101100101011",
 	"11010010111011",
@@ -99,7 +100,7 @@ const char* shape_codes[][Const::max_shapes] = {
 	"1101000100100000100000111",
 	"1110001000100000001010011" },
 	{
-		"11001011",		// 16x5
+		"11001011",		// 2 16x5
 		"101010011",
 	"100110101",
 	"1110000101",
@@ -117,7 +118,7 @@ const char* shape_codes[][Const::max_shapes] = {
 	"10010000100000000000011" },
 
 	{
-		"11101011", // 16x6
+		"11101011", // 3 16x6
 "110100111",
 "11001000111",
 "1100001001011",
@@ -135,7 +136,7 @@ const char* shape_codes[][Const::max_shapes] = {
 "101000010000000000010011"
 	},
 
-	{ "1110010111", // 16x7
+	{ "1110010111", // 4 16x7
 	"11001101011",
 	"1101001000111",
 	"11100010010011",
@@ -154,7 +155,7 @@ const char* shape_codes[][Const::max_shapes] = {
 
 	},
 	{
-		"101011",		// 16x4
+		"101011",		// 5 16x4
 		"110011",
 	"110000101",
 	"1001000011",
@@ -170,7 +171,18 @@ const char* shape_codes[][Const::max_shapes] = {
 	"101000000000000011",
 	"100010000000000000101",
 	"1000100000000000100001"
-	}
+	},
+	{
+		"111010110110111","111001010101001111","1110110010001101011","11110010000100100010111",0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, // 6 4x11
+{	"1110101101111","1110110100010111","10110110001001000111","111010001000010010111",0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, // 7 4x10
+//{"1111011101011","1110100111111",0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, // 8 2x10
+//{"1110101101111","11101001100010111",0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, // 8 2x10
+ { "111101110111","111011010010111" },	// 8 2x10 iedera
+//{"111011111","11011010111","111101001011","1110011001011","1110001101101","1101000111011","1101010100111","11101001000111","110100100101011","111001010001011","110011000100111","1110100001010011","1101000100100111","11010100010001011","11100010100001011","11101000000010001011"}, // 9 16x8
+{"1011110111","110100100010111","11001011111","101110001111","11011101100001","1111010010101","111001001001011","10101001101011","111101010011","1111000010000111","1100011011011","1101010000011011","1110001010101001","110011000110011","11011010001101","1101001100010011"},// 9 16x8 iedera
+{"111111",0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,0 }, // 10 1x6
+{"11111",0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,0 } // 11 1x5
+
 };
 
 shape_config shapes;
