@@ -1,19 +1,19 @@
 /****
-Copyright (c) 2016, Benjamin Buchfink
-All rights reserved.
+DIAMOND protein aligner
+Copyright (C) 2013-2017 Benjamin Buchfink <buchfink@gmail.com>
 
-Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
+This program is free software: you can redistribute it and/or modify
+it under the terms of the GNU Affero General Public License as
+published by the Free Software Foundation, either version 3 of the
+License, or (at your option) any later version.
 
-1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU Affero General Public License for more details.
 
-2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+You should have received a copy of the GNU Affero General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
 #include "../util/command_line_parser.h"
@@ -31,6 +31,7 @@ LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR P
 #include "../data/sorted_list.h"
 #include "../basic/translate.h"
 #include "../dp/dp.h"
+#include "masking.h"
 
 Config config;
 
@@ -53,7 +54,9 @@ Config::Config(int argc, const char **argv)
 		.add_command("modelsim", "")
 		.add_command("match-file-stat", "")
 		.add_command("model-seqs", "")
-		.add_command("opt", "");
+		.add_command("opt", "")
+		.add_command("mask", "")
+		.add_command("fastq2fasta", "");
 
 	Options_group general("General options");
 	general.add()
@@ -91,6 +94,7 @@ Config::Config(int argc, const char **argv)
 \tppos means Percentage of positive - scoring matches\n\
 \tqframe means Query frame\n\
 \tbtop means Blast traceback operations(BTOP)\n\
+\tstaxids means unique Subject Taxonomy ID(s), separated by a ';' (in numerical order)\n\
 \tstitle means Subject Title\n\
 \tsalltitles means All Subject Title(s), separated by a '<>'\n\
 \tqcovhsp means Query Coverage Per HSP\n\
@@ -112,7 +116,7 @@ Config::Config(int argc, const char **argv)
 		("max-target-seqs", 'k', "maximum number of target sequences to report alignments for", max_alignments, uint64_t(25))
 		("top", 0, "report alignments within this percentage range of top alignment score (overrides --max-target-seqs)", toppercent, 100.0)
 		("compress", 0, "compression for output files (0=none, 1=gzip)", compression)
-		("evalue", 'e', "maximum e-value to report alignments", max_evalue, 0.001)
+		("evalue", 'e', "maximum e-value to report alignments (default=0.001)", max_evalue, 0.001)
 		("min-score", 0, "minimum bit score to report alignments (overrides e-value setting)", min_bit_score)
 		("id", 0, "minimum identity% to report an alignment", min_id)
 		("query-cover", 0, "minimum query cover% to report an alignment", query_cover)
@@ -122,20 +126,23 @@ Config::Config(int argc, const char **argv)
 		("block-size", 'b', "sequence block size in billions of letters (default=2.0)", chunk_size)
 		("index-chunks", 'c', "number of chunks for index processing", lowmem)
 		("tmpdir", 't', "directory for temporary files", tmpdir)
-		("gapopen", 0, "gap open penalty (default=11 for protein)", gap_open, -1)
-		("gapextend", 0, "gap extension penalty (default=1 for protein)", gap_extend, -1)
+		("gapopen", 0, "gap open penalty", gap_open, -1)
+		("gapextend", 0, "gap extension penalty", gap_extend, -1)
 		("matrix", 0, "score matrix for protein alignment (default=BLOSUM62)", matrix, string("blosum62"))
 		("custom-matrix", 0, "file containing custom scoring matrix", matrix_file)
 		("lambda", 0, "lambda parameter for custom matrix", lambda)
 		("K", 0, "K parameter for custom matrix", K)
 		("comp-based-stats", 0, "enable composition based statistics (0/1=default)", comp_based_stats, 1u)
-		("seg", 0, "enable SEG masking of queries (yes/no)", seg)
+		("masking", 0, "enable masking of low complexity regions (0/1=default)", masking, 1)
+		//("seg", 0, "enable SEG masking of queries (yes/no)", seg)
 		("query-gencode", 0, "genetic code to use to translate query (see user manual)", query_gencode, 1u)
-		("salltitles", 0, "print full subject titles in output files", salltitles)
-		("no-self-hits", 0, "suppress reporting of identical self hits", no_self_hits);
+		("salltitles", 0, "include full subject titles in DAA file", salltitles)
+		("no-self-hits", 0, "suppress reporting of identical self hits", no_self_hits)
+		("taxonmap", 0, "protein accession to taxid mapping file", prot_accession2taxid);
 
 	Options_group advanced("Advanced options");
 	advanced.add()
+		("algo", 0, "Seed search algorithm (0=double-indexed/1=query-indexed)", algo, -1)
 		("bin", 0, "number of query bins for seed search", query_bins, 16u)
 		("min-orf", 'l', "ignore translated sequences without an open reading frame of at least this length", run_len)
 		("freq-sd", 0, "number of standard deviations for ignoring frequent seeds", freq_sd, 0.0)
@@ -150,13 +157,11 @@ Config::Config(int argc, const char **argv)
 		("shapes", 's', "number of seed shapes (0 = all available)", shapes)
 		("shape-mask", 0, "seed shapes", shape_mask)
 		("index-mode", 0, "index mode (0=4x12, 1=16x9)", index_mode)
-		("fetch-size", 0, "trace point fetch size", fetch_size, 4096u)
-		("rank-factor", 0, "include subjects within this range of max-target-seqs", rank_factor, 2.0)
-		("rank-ratio", 0, "include subjects within this ratio of last hit", rank_ratio, 0.6)
+		("rank-ratio", 0, "include subjects within this ratio of last hit (stage 1)", rank_ratio, -1.0)
+		("rank-ratio2", 0, "include subjects within this ratio of last hit (stage 2)", rank_ratio2, -1.0)
 		("max-hsps", 0, "maximum number of HSPs per subject sequence to save for each query", max_hsps, 1u)
 		("dbsize", 0, "effective database size (in letters)", db_size)
-		("no-auto-append", 0, "disable auto appending of DAA and DMND file extensions", no_auto_append)
-		("target-fetch-size", 0, "number of target sequences to fetch for seed extension", target_fetch_size, 4u);
+		("no-auto-append", 0, "disable auto appending of DAA and DMND file extensions", no_auto_append);
 
 	Options_group view_options("View options");
 	view_options.add()
@@ -174,16 +179,13 @@ Config::Config(int argc, const char **argv)
 		("slow-search", 0, "", slow_search)
 		("ht", 0, "", ht_mode)
 		("old-freq", 0, "", old_freq)
-		("qp", 0, "", query_parallel)
 		("match1", 0, "", match_file1)
 		("match2", 0, "", match_file2)
 		("max-hits", 'C', "maximum number of hits to consider for one seed", hit_cap)
 		("seed-freq", 0, "maximum seed frequency", max_seed_freq, -15.0)
-		("space-penalty", 0, "", space_penalty, 4.9)
-		("min-diag-score", 0, "", min_diag_score, 10.5)
+		("space-penalty", 0, "", space_penalty, 0.5)
 		("reverse", 0, "", reverse)
 		("neighborhood-score", 0, "", neighborhood_score)
-		("algo", 0, "", algo, 0u)
 		("seed-weight", 'w', "", seed_weight, 7u)
 		("very-sensitive", 0, "", mode_very_sensitive)
 		("idl", 0, "", id_left)
@@ -196,24 +198,39 @@ Config::Config(int argc, const char **argv)
 		("rho", 0, "", rho, 0.99)
 		("p_best", 0, "", p_best, 0.05)
 		("d_exp", 0, "", d_exp, 1.0)
-		("d_new", 0, "", d_new, 1.0);
+		("d_new", 0, "", d_new, 1.0)
+		("score-estimate-factor", 0, "", score_estimate_factor, 0.0)
+		("diag-min-estimate", 0, "", diag_min_estimate, 17)
+		("qfilt", 0, "", qfilt)
+		("sfilt", 0, "", sfilt)
+		("path-cutoff", 0, "", path_cutoff, 0.92)
+		("sw", 0, "", use_smith_waterman)
+		("superblock", 0, "", superblock, 128)
+		("max-cells", 0, "", max_cells, 10000000u)
+		("lb", 0, "", load_balancing, (unsigned)Config::query_parallel)
+		("ext", 0, "", ext, (int)Config::greedy)
+		("br", 0, "", benchmark_ranking)
+		("log-query", 0, "", log_query)
+		("log-subject", 0, "", log_subject)
+		("palign", 0, "", threads_align)
+		("score-ratio", 0, "", score_ratio, 0.9)
+		("fetch-size", 0, "trace point fetch size", fetch_size, 4096u)
+		("target-fetch-size", 0, "number of target sequences to fetch for seed extension", target_fetch_size, 4u)
+		("small-query", 0, "", small_query)
+		("hashed-seeds", 0, "", hashed_seeds);
 		
 	parser.add(general).add(makedb).add(aligner).add(advanced).add(view_options).add(getseq_options).add(hidden_options);
 	parser.store(argc, argv, command);
 
 	switch (command) {
 	case Config::makedb:
-		if (input_ref_file == "")
-			throw std::runtime_error("Missing parameter: input file (--in)");
 		if (database == "")
 			throw std::runtime_error("Missing parameter: database file (--db/-d)");
 		if (chunk_size != 0.0)
 			throw std::runtime_error("Invalid option: --block-size/-b. Block size is set for the alignment commands.");
 		break;
 	case Config::blastp:
-	case Config::blastx:
-		if (query_file == "")
-			throw std::runtime_error("Missing parameter: query file (--query/-q)");
+	case Config::blastx:	
 		if (database == "")
 			throw std::runtime_error("Missing parameter: database file (--db/-d)");
 		if (daa_file.length() > 0) {
@@ -247,7 +264,7 @@ Config::Config(int argc, const char **argv)
 	else if (verbose)
 		verbosity = 2;
 	else if (((command == Config::view || command == blastx || command == blastp) && output_file == "")
-		|| command == Config::version || command == getseq)
+		|| command == Config::version || command == getseq || command == fastq2fasta)
 		verbosity = 0;
 	else
 		verbosity = 1;
@@ -280,6 +297,7 @@ Config::Config(int argc, const char **argv)
 	}
 
 	message_stream << Const::program_name << " v" << Const::version_string << "." << (unsigned)Const::build_version << " | by Benjamin Buchfink <buchfink@gmail.com>" << endl;
+	message_stream << "Licensed under the GNU AGPL <https://www.gnu.org/licenses/agpl.txt>" << endl;
 	message_stream << "Check http://github.com/bbuchfink/diamond for updates." << endl << endl;
 #ifndef NDEBUG
 	verbose_stream << "Assertions enabled." << endl;
@@ -296,52 +314,45 @@ Config::Config(int argc, const char **argv)
 		;
 	}
 
-	if (command == Config::blastp || command == Config::blastx || command == Config::benchmark || command == Config::model_sim || command == Config::opt) {
-		if (tmpdir == "")
-			tmpdir = extract_dir(output_file);
-		if (gap_open == -1)
-			gap_open = 11;
-		if (gap_extend == -1)
-			gap_extend = 1;
+	switch (command) {
+	case Config::blastp:
+	case Config::blastx:
+	case Config::benchmark:
+	case Config::model_sim:
+	case Config::opt:
+	case Config::mask:
+	case Config::makedb:
 		if (matrix_file == "")
 			score_matrix = Score_matrix(to_upper_case(matrix), gap_open, gap_extend, reward, penalty);
 		else {
 			if (lambda == 0 || K == 0)
 				throw std::runtime_error("Custom scoring matrices require setting the --lambda and --K options.");
+			if (gap_open == -1 || gap_extend == -1)
+				throw std::runtime_error("Custom scoring matrices require setting the --gapopen and --gapextend options.");
 			score_matrix = Score_matrix(matrix_file, lambda, K, gap_open, gap_extend);
 		}
 		message_stream << "Scoring parameters: " << score_matrix << endl;
-		raw_ungapped_xdrop = score_matrix.rawscore(ungapped_xdrop);
-		min_diag_raw_score = score_matrix.rawscore(min_diag_score);
-		raw_space_penalty = score_matrix.rawscore(space_penalty, double());
-		log_stream << "Min_diag_score=" << min_diag_raw_score << " space_penalty=" << raw_space_penalty << endl;
+		if (masking == 1)
+			Masking::instance = auto_ptr<Masking>(new Masking(score_matrix));
+	}
+
+	if (command == Config::blastp || command == Config::blastx || command == Config::benchmark || command == Config::model_sim || command == Config::opt
+		|| command == Config::mask) {
+		if (tmpdir == "")
+			tmpdir = extract_dir(output_file);
+		
 		init_cbs();
+		raw_ungapped_xdrop = score_matrix.rawscore(ungapped_xdrop);
 
-		if (seg == "" && command == blastx)
-			seg = "yes";
-		verbose_stream << "SEG masking = " << (seg == "yes") << endl;
-		have_ssse3 = check_SSSE3();
-		if (have_ssse3)
-			verbose_stream << "SSSE3 enabled." << endl;
-		verbose_stream << "Reduction: " << Reduction::reduction << endl;
-
-		if (mode_more_sensitive) {
-			set_option(index_mode, 1u);
-			set_option(freq_sd, 200.0);
-		}
-		else if (mode_sensitive) {
-			set_option(index_mode, 1u);
-			set_option(freq_sd, 10.0);
-		}
-		else {
-			set_option(index_mode, 0u);
-			set_option(freq_sd, 50.0);
-		}
-
-		verbose_stream << "Seed frequency SD: " << freq_sd << endl;
-		::shapes = shape_config(index_mode, shapes, shape_mask);
-		verbose_stream << "Shape configuration: " << ::shapes << endl;
-		seed_anchor = std::min(::shapes[0].length_ - 1, 8u);
+#ifdef __SSSE3__
+		verbose_stream << "SSSE3 enabled." << endl;
+#endif
+#ifdef __SSE4_1__
+		verbose_stream << "SSE4.1 enabled." << endl;
+#endif
+#ifdef __POPCNT__
+		verbose_stream << "POPCNT enabled." << endl;
+#endif
 
 		message_stream << "#Target sequences to report alignments for: ";
 		if (max_alignments == 0) {
