@@ -1,19 +1,19 @@
 /****
-Copyright (c) 2016, Benjamin Buchfink
-All rights reserved.
+DIAMOND protein aligner
+Copyright (C) 2013-2017 Benjamin Buchfink <buchfink@gmail.com>
 
-Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
+This program is free software: you can redistribute it and/or modify
+it under the terms of the GNU Affero General Public License as
+published by the Free Software Foundation, either version 3 of the
+License, or (at your option) any later version.
 
-1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU Affero General Public License for more details.
 
-2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+You should have received a copy of the GNU Affero General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
 #ifndef CONFIG_H_
@@ -69,9 +69,7 @@ struct Config
 	int		gap_open;
 	int		gap_extend;
 	string	matrix;
-	string	seg;
 	bool debug_log, verbose, quiet;
-	bool		have_ssse3;
 	bool		salltitles;
 	int		reward;
 	int		penalty;
@@ -94,12 +92,11 @@ struct Config
 	bool extend_all;
 	bool slow_search;
 	vector<string> seq_no;
-	double rank_factor;
 	double rank_ratio;
+	double rank_ratio2;
 	bool ht_mode;
 	bool old_freq;
 	double freq_sd;
-	bool query_parallel;
 	unsigned target_fetch_size;
 	bool mode_more_sensitive;
 	string matrix_file;
@@ -108,9 +105,7 @@ struct Config
 	unsigned seed_anchor;
 	unsigned query_gencode;
 	string unaligned;
-	double space_penalty, raw_space_penalty;
-	double min_diag_score;
-	int min_diag_raw_score;
+	double space_penalty;
 	bool new_prefilter;
 	bool reverse;
 	unsigned comp_based_stats;
@@ -128,15 +123,39 @@ struct Config
 	double rho;
 	double p_best;
 	double d_exp, d_new;
+	double score_estimate_factor;
+	int diag_min_estimate;
+	string qfilt, sfilt;
+	double path_cutoff;
+	bool use_smith_waterman;
+	string prot_accession2taxid;
+	int superblock;
+	unsigned max_cells;
+	int masking;
+	bool benchmark_ranking;
+	bool log_query;
+	bool log_subject;
+	unsigned threads_align;
+	double score_ratio;
+	bool small_query;
+	bool hashed_seeds;
 
 	enum {
 		makedb = 0, blastp = 1, blastx = 2, view = 3, help = 4, version = 5, getseq = 6, benchmark = 7, random_seqs = 8, compare = 9, sort = 10, roc = 11, db_stat = 12, model_sim = 13,
-		match_file_stat = 14, model_seqs = 15, opt = 16
+		match_file_stat = 14, model_seqs = 15, opt = 16, mask = 17, fastq2fasta=18
 	};
 	unsigned	command;
 
-	enum { double_indexed = 0, subject_indexed = 1 };
-	unsigned algo;
+	enum { double_indexed = 0, query_indexed = 1, subject_indexed = 2 };
+	int algo;
+
+	enum { query_parallel = 0, target_parallel = 1 };
+	unsigned load_balancing;
+
+	enum {
+		swipe = 0, greedy = 1, floating_xdrop = 4, more_greedy = 2, most_greedy=3,
+	};
+	int ext;
 
 	Config() {}
 	Config(int argc, const char **argv);
