@@ -1,19 +1,19 @@
 /****
-Copyright (c) 2014-2016, University of Tuebingen, Benjamin Buchfink
-All rights reserved.
+DIAMOND protein aligner
+Copyright (C) 2013-2017 Benjamin Buchfink <buchfink@gmail.com>
 
-Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
+This program is free software: you can redistribute it and/or modify
+it under the terms of the GNU Affero General Public License as
+published by the Free Software Foundation, either version 3 of the
+License, or (at your option) any later version.
 
-1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU Affero General Public License for more details.
 
-2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+You should have received a copy of the GNU Affero General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
 #ifndef MATCH_H_
@@ -81,6 +81,15 @@ struct Diagonal_segment
 	{
 		return i - j;
 	}
+	Diagonal_segment intersect(const Diagonal_segment &x) const
+	{
+		if (diag() != x.diag())
+			return Diagonal_segment();
+		else {
+			const interval q = ::intersect(query_range(), x.query_range());
+			return Diagonal_segment(q.begin_, ::intersect(subject_range(), x.subject_range()).begin_, q.length(), 0);
+		}
+	}
 	bool is_enveloped(const Diagonal_segment &x) const
 	{
 		return score <= x.score
@@ -99,18 +108,32 @@ struct Diagonal_segment
 	{
 		return i + len <= rhs.i && j + len <= rhs.j;
 	}
+	bool operator==(const Diagonal_segment &rhs) const
+	{
+		return i == rhs.i && j == rhs.j && len == rhs.len;
+	}
 	static bool cmp_subject(const Diagonal_segment &x, const Diagonal_segment &y)
 	{
-		return x.j < y.j;
+		return x.j < y.j || (x.j == y.j && x.i < y.i);
 	}
 	static bool cmp_subject_end(const Diagonal_segment &x, const Diagonal_segment &y)
 	{
 		return x.subject_end() < y.subject_end();
 	}
+	static bool cmp_heuristic(const Diagonal_segment &x, const Diagonal_segment &y)
+	{
+		return (x.subject_end() < y.subject_end() && x.j < y.j)
+			|| (x.j - y.j < y.subject_end() - x.subject_end());
+	}
 	friend int abs_shift(const Diagonal_segment &x, const Diagonal_segment &y)
 	{
 		return abs(x.diag() - y.diag());
 	}
+	friend std::ostream& operator<<(std::ostream &s, const Diagonal_segment &d)
+	{
+		s << "i=" << d.i << " j=" << d.j << " l=" << d.len << " score=" << d.score;
+		return s;
+	}
 	int i, j, len, score;
 };
 
@@ -118,7 +141,7 @@ struct Intermediate_record;
 
 struct Hsp_data
 {
-	Hsp_data():
+	Hsp_data() :
 		score(0),
 		frame(0),
 		length(0),
@@ -126,9 +149,10 @@ struct Hsp_data
 		mismatches(0),
 		positives(0),
 		gap_openings(0),
-		gaps(0)
+		gaps(0),
+		sw_score(0)
 	{}
-	Hsp_data(int score):
+	Hsp_data(int score) :
 		score(unsigned(score)),
 		frame(0),
 		length(0),
@@ -136,7 +160,8 @@ struct Hsp_data
 		mismatches(0),
 		positives(0),
 		gap_openings(0),
-		gaps(0)
+		gaps(0),
+		sw_score(0)
 	{}
 	Hsp_data(const Intermediate_record &r, unsigned query_source_len);
 	struct Iterator
@@ -232,10 +257,16 @@ struct Hsp_data
 	{
 		return (double)subject_range.length() * 100 / subject_len;
 	}
+	static bool cmp_query_pos(const Hsp_data &x, const Hsp_data &y)
+	{
+		return x.query_range.begin_ < y.query_range.begin_;
+	}
 	bool pass_through(const Diagonal_segment &d) const;
 	bool is_weakly_enveloped(const Hsp_data &j) const;
+	std::pair<int, int> diagonal_bounds() const;
 	void merge(const Hsp_data &right, const Hsp_data &left, unsigned query_anchor, unsigned subject_anchor);
-	unsigned score, frame, length, identities, mismatches, positives, gap_openings, gaps;
+	unsigned score, frame, length, identities, mismatches, positives, gap_openings, gaps, sw_score;
+	float time;
 	interval query_source_range, query_range, subject_range;
 	Packed_transcript transcript;
 };
@@ -329,6 +360,14 @@ struct Hsp_context
 	{
 		return score_matrix.bitscore(score());
 	}
+	double sw_score() const
+	{
+		return score_matrix.bitscore(hsp_.sw_score);
+	}
+	double time() const
+	{
+		return hsp_.time;
+	}
 	unsigned frame() const
 	{ return hsp_.frame; }
 	unsigned length() const
