@@ -1,21 +1,19 @@
 /****
-Copyright (c) 2014, University of Tuebingen
-All rights reserved.
+DIAMOND protein aligner
+Copyright (C) 2013-2017 Benjamin Buchfink <buchfink@gmail.com>
 
-Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
+This program is free software: you can redistribute it and/or modify
+it under the terms of the GNU Affero General Public License as
+published by the Free Software Foundation, either version 3 of the
+License, or (at your option) any later version.
 
-1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU Affero General Public License for more details.
 
-2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-****
-Author: Benjamin Buchfink
+You should have received a copy of the GNU Affero General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
 #ifndef REDUCTION_H_
@@ -41,6 +39,7 @@ struct Reduction
 		map_[(long)value_traits.mask_char] = value_traits.mask_char;
 		const vector<string> tokens(tokenize(definition_string, " "));
 		size_ = (unsigned)tokens.size();
+		bit_size_ = (uint64_t)ceil(log(size_) / log(2));
 		for (unsigned i = 0; i<size_; ++i)
 			for (unsigned j = 0; j<tokens[i].length(); ++j) {
 				const char ch = tokens[i][j];
@@ -55,6 +54,11 @@ struct Reduction
 		return size_;
 	}
 
+	uint64_t bit_size() const
+	{
+		return bit_size_;
+	}
+
 	unsigned operator()(Letter a) const
 	{
 		return map_[(long)a];
@@ -96,6 +100,7 @@ private:
 	char map8_[256] __attribute__((aligned(16)));
 #endif
 	unsigned size_;
+	uint64_t bit_size_;
 
 };
 
@@ -103,4 +108,4 @@ private:
 #include "../../../extra/reduction.h"
 #endif
 
-#endif /* REDUCTION_H_ */
+#endif /* REDUCTION_H_ */
\ No newline at end of file
