@@ -1,23 +1,25 @@
 /****
-Copyright (c) 2016, Benjamin Buchfink
-All rights reserved.
+DIAMOND protein aligner
+Copyright (C) 2013-2017 Benjamin Buchfink <buchfink@gmail.com>
 
-Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
+This program is free software: you can redistribute it and/or modify
+it under the terms of the GNU Affero General Public License as
+published by the Free Software Foundation, either version 3 of the
+License, or (at your option) any later version.
 
-1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU Affero General Public License for more details.
 
-2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+You should have received a copy of the GNU Affero General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
+#include <set>
 #include "../basic/match.h"
 #include "output_format.h"
+#include "../data/taxonomy.h"
 
 const char* Blast_tab_format::field_str[] = {
 	"qseqid",		// 0 means Query Seq - id
@@ -65,7 +67,9 @@ const char* Blast_tab_format::field_str[] = {
 	"qcovs",		// 42 means Query Coverage Per Subject
 	"qcovhsp",		// 43 means Query Coverage Per HSP
 	"qcovus",		// 44 means Query Coverage Per Unique Subject(blastn only)
-	"qtitle"		// 45 means Query title
+	"qtitle",		// 45 means Query title
+	"swdiff",		// 46
+	"time"			// 47
 };
 
 Blast_tab_format::Blast_tab_format() :
@@ -81,12 +85,26 @@ Blast_tab_format::Blast_tab_format() :
 		int j = get_idx(field_str, sizeof(field_str) / sizeof(field_str[0]), i->c_str());
 		if(j == -1)
 			throw std::runtime_error(string("Invalid output field: ") + *i);
+		if (j == 34 && config.prot_accession2taxid.empty())
+			throw std::runtime_error("staxids output field requires setting the --taxonmap parameter.");
 		fields.push_back(j);
-		if (j == 6 || j == 39 || j == 40)
+		if (j == 6 || j == 39 || j == 40 || j == 34)
 			config.salltitles = true;
 	}
 }
 
+void print_staxids(Text_buffer &out, const char *id)
+{
+	const vector<string> t(tokenize(id, "\1"));	
+	std::set<unsigned> taxons;
+	for (vector<string>::const_iterator i = t.begin(); i < t.end(); ++i)
+		taxons.insert(taxonomy.get(Taxonomy::Accession(*i)));
+	std::set<unsigned>::const_iterator i = taxons.begin();
+	out << *(i++);
+	for (; i != taxons.end(); ++i)
+		out << ';' << *i;
+}
+
 void Blast_tab_format::print_match(const Hsp_context& r, Text_buffer &out) const
 {
 	for (vector<unsigned>::const_iterator i = fields.begin(); i != fields.end(); ++i) {
@@ -98,10 +116,10 @@ void Blast_tab_format::print_match(const Hsp_context& r, Text_buffer &out) const
 			out << r.source_query.length();
 			break;
 		case 5:
-			this->print_salltitles(out, r.subject_name, false, false);
+			print_title(out, r.subject_name, false, false, "<>");
 			break;
 		case 6:
-			this->print_salltitles(out, r.subject_name, false, true);
+			print_title(out, r.subject_name, false, true, "<>");
 			break;
 		case 12:
 			out << r.subject_len;
@@ -202,11 +220,14 @@ void Blast_tab_format::print_match(const Hsp_context& r, Text_buffer &out) const
 				out << n_matches;
 		}
 			break;
+		case 34:
+			print_staxids(out, r.subject_name);
+			break;
 		case 39:
-			this->print_salltitles(out, r.subject_name, true, false);
+			print_title(out, r.subject_name, true, false, "<>");
 			break;
 		case 40:
-			this->print_salltitles(out, r.subject_name, true, true);
+			print_title(out, r.subject_name, true, true, "<>");
 			break;
 		case 43:
 			out << (double)r.query_source_range().length()*100.0 / r.source_query.length();
@@ -214,6 +235,12 @@ void Blast_tab_format::print_match(const Hsp_context& r, Text_buffer &out) const
 		case 45:
 			out << r.query_name;
 			break;
+		case 46:
+			out << r.sw_score() - r.bit_score();
+			break;
+		case 47:
+			out << r.time();
+			break;
 		default:
 			throw std::runtime_error("Invalid output field");
 		}
@@ -263,6 +290,7 @@ void Blast_tab_format::print_query_intro(size_t query_num, const char *query_nam
 				out << "-1";
 				break;			
 			case 31:
+			case 34:
 				out << '0';
 				break;
 			case 45:
